@@ -1,6 +1,7 @@
 /*
  * syncml plugin - A syncml plugin for OpenSync
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008-2009  Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,33 +22,12 @@
 #include "syncml_common.h"
 #include "syncml_callbacks.h"
 #include "syncml_devinf.h"
-#include "syncml_vformat.h"
 #include "syncml_ds_client.h"
 #include "syncml_ds_server.h"
 
-#include <opensync/plugin/opensync_sink.h>
+#include <opensync/opensync-plugin.h>
 #include <libsyncml/data_sync_api/callbacks.h>
 
-GList *g_list_add(GList *databases, void *database)
-{
-    osync_trace(TRACE_ENTRY, "%s", __func__);
-
-    // if we find the item in the list
-    // then we only return and do nothing
-
-    if (g_list_find(databases, database) != NULL)
-    {
-        osync_trace(TRACE_EXIT, "%s - the item is an element of the list", __func__);
-        return databases;
-    }
-
-    // add the item to the list
-
-    GList *result = g_list_append(databases, database);
-    osync_trace(TRACE_EXIT, "%s - add a new list item %p", __func__, database);
-    return result;
-}
-
 void syncml_free_database(SmlDatabase *database)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, database);
@@ -62,6 +42,17 @@ void syncml_free_database(SmlDatabase *database)
 		osync_objformat_unref(database->objformat);
 		database->objformat = NULL;
 	}
+
+	if (database->alert_type_mutex) {
+		g_mutex_free(database->alert_type_mutex);
+		database->alert_type_mutex = NULL;
+	}
+
+	if (database->alert_type_cond) {
+		g_cond_free(database->alert_type_cond);
+		database->alert_type_cond = NULL;
+	}
+
 	if (database->sink) {
 		osync_objtype_sink_unref(database->sink);
 		database->sink = NULL;
@@ -101,6 +92,11 @@ void syncml_free_database(SmlDatabase *database)
 		osync_error_set(&error, OSYNC_ERROR_GENERIC, "%s - syncModeCtx context discovered on finalize", __func__);
 		report_error_on_context(&(database->syncModeCtx), &error, TRUE);
 	}
+	if (database->connectCtx) {
+		OSyncError *error = NULL;
+		osync_error_set(&error, OSYNC_ERROR_GENERIC, "%s - connectCtx context discovered on finalize", __func__);
+		report_error_on_context(&(database->connectCtx), &error, TRUE);
+	}
 	if (database->getChangesCtx) {
 		OSyncError *error = NULL;
 		osync_error_set(&error, OSYNC_ERROR_GENERIC, "%s - getChangesCtx context discovered on finalize", __func__);
@@ -186,23 +182,65 @@ OSyncChangeType _to_osync_changetype(SmlChangeType type)
 	return OSYNC_CHANGE_TYPE_UNKNOWN;
 }
 
-void sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+void syncml_connect_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data)
 {
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+
+
+        SmlDatabase *database = (SmlDatabase*)data;
+
+	g_mutex_lock(database->alert_type_mutex);
+
+	database->slowsync = slow_sync;
+
+	g_cond_signal(database->alert_type_cond);
+	g_mutex_unlock(database->alert_type_mutex);
+
 	osync_context_report_success(ctx);
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
 }
 
-void disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+void sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, ctx, data);
+	SmlPluginEnv *env = (SmlPluginEnv *)data;
+	g_assert(env);
+
+	osync_context_report_success(ctx);
+
+	/* disconnect is allowed now */
+	env->waitForDisconnect = TRUE;
+
+	osync_trace(TRACE_EXIT, "%s", __func__, ctx);
+}
+
+void disconnect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, ctx);
 	SmlPluginEnv *env = (SmlPluginEnv *)data;
 	g_assert(env);
 
-	if ((env->state1 >= SML_DATA_SYNC_EVENT_DISCONNECT &&
-	     (!env->dsObject2 ||
-	      env->state2 >= SML_DATA_SYNC_EVENT_DISCONNECT ||
-	      env->state2 < SML_DATA_SYNC_EVENT_CONNECT)
-	    ) ||
-	    env->state1 < SML_DATA_SYNC_EVENT_CONNECT)
+	SmlBool waitForConnectDone = FALSE;
+	if (env->databases &&
+	    ((SmlDatabase *)env->databases->data)->alert_type_mutex &&
+	    !((SmlDatabase *)env->databases->data)->connectCtx) {
+		/* the datastores are waiting for the approved alert types */
+		waitForConnectDone = TRUE;
+	}
+
+	if (!waitForConnectDone &&
+	    env->waitForDisconnect &&
+	    (
+	     (env->state1 >= SML_DATA_SYNC_EVENT_DISCONNECT &&
+	      (!env->dsObject2 ||
+	       env->state2 >= SML_DATA_SYNC_EVENT_DISCONNECT ||
+	       env->state2 < SML_DATA_SYNC_EVENT_CONNECT)
+	     ) ||
+	     env->state1 < SML_DATA_SYNC_EVENT_CONNECT
+            )
+	   )
 	{
 		/* The disconnect already happened or
 		 * the session never existed (connect error). */
@@ -210,6 +248,41 @@ void disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	} else {
 		env->disconnectCtx = ctx;
 		osync_context_ref(env->disconnectCtx);
+
+		/* If the disconnect is part of an abort and
+		 * there is a hanging alert type callback
+		 * then the condition must be signaled to
+		 * abort the connection.
+		 */
+		GList *list = env->databases;
+		for (; list; list = list->next) {
+			SmlDatabase *database = list->data;
+			if (database->alert_type_mutex) {
+				osync_trace(TRACE_INTERNAL, "%s: signal alert_type approval", __func__);
+				g_mutex_lock(database->alert_type_mutex);
+				g_cond_signal(database->alert_type_cond);
+				g_mutex_unlock(database->alert_type_mutex);
+				osync_trace(TRACE_INTERNAL, "%s: signalled alert_type approval", __func__);
+			}
+		}
+
+		if (!env->waitForDisconnect) {
+			/* this is an abort */
+			osync_trace(TRACE_INTERNAL, "%s: abort detected", __func__);
+			SmlError *error = NULL;
+			if (env->dsObject2 && env->state2 > SML_DATA_SYNC_EVENT_CONNECT)
+			{
+				smlDataSyncAbort(env->dsObject2, &error);
+			} else {
+				smlDataSyncAbort(env->dsObject1, &error);
+			}
+			if (error != NULL)
+			{
+				OSyncError *oserror = NULL;
+				osync_error_set(&oserror, OSYNC_ERROR_GENERIC, "%s", smlErrorPrint(&error));
+				report_error_on_context(&(env->disconnectCtx), &oserror, TRUE);
+			}
+		}
 	}
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -254,19 +327,10 @@ void finalize(void *data)
 
 	/* plugin config */
 
-	if (env->anchor_path)
-		safe_cfree(&(env->anchor_path));
-	if (env->devinf_path)
-		safe_cfree(&(env->devinf_path));
 	osync_trace(TRACE_INTERNAL, "%s - plugin configuration cleaned", __func__);
 
 	/* Signal forgotten contexts */
 
-	if (env->connectCtx) {
-		OSyncError *oerror = NULL;
-		osync_error_set(&oerror, OSYNC_ERROR_GENERIC, "%s - detected forgotten connect context", __func__);
-		report_error_on_context(&(env->connectCtx), &oerror, TRUE);
-	}
 	if (env->disconnectCtx) {
 		OSyncError *oerror = NULL;
 		osync_error_set(&oerror, OSYNC_ERROR_GENERIC, "%s - detected forgotten disconnect context", __func__);
@@ -309,15 +373,6 @@ unsigned int get_num_changes(OSyncChange **changes)
     return num;
 }
 
-SmlDatabase *get_database_from_plugin_info(OSyncPluginInfo *info)
-{
-    osync_trace(TRACE_ENTRY, "%s", __func__);
-    OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-    SmlDatabase *database = osync_objtype_sink_get_userdata(sink);
-    osync_trace(TRACE_EXIT, "%s - %s", __func__, database->url);
-    return database;
-} 
-
 void safe_cfree(char **address)
 {
     safe_free((gpointer *)address);
@@ -392,11 +447,12 @@ error:
 	return NULL;
 }
 
-void syncml_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+void syncml_connect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 
-	SmlPluginEnv *env = (SmlPluginEnv *)data;
+	SmlDatabase *database = (SmlDatabase *)data;
+	SmlPluginEnv *env = database->env;
 	SmlError *error = NULL;
 	OSyncError *oserror = NULL;
 
@@ -405,13 +461,18 @@ void syncml_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 	 * then the connect context is signalled.
 	 */
 
-	env->connectCtx = ctx;
-	osync_context_ref(env->connectCtx);
-
-	if (!smlDataSyncInit(env->dsObject1, &error))
-		goto error;
-	if (!smlDataSyncRun(env->dsObject1, &error))
-		goto error;
+	database->connectCtx = ctx;
+	osync_context_ref(database->connectCtx);
+	if (g_atomic_int_dec_and_test(&(env->toBeConnected))) {
+		/* all database received the connect callback */
+		osync_trace(TRACE_INTERNAL, "%s: executing connect", __func__);
+		if (!smlDataSyncInit(env->dsObject1, &error))
+			goto error;
+		if (!smlDataSyncRun(env->dsObject1, &error))
+			goto error;
+	} else {
+		osync_trace(TRACE_INTERNAL, "%s: just another connect", __func__);
+	}
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return;
@@ -419,7 +480,7 @@ error:
 	osync_error_set(&oserror, OSYNC_ERROR_GENERIC, "%s", smlErrorPrint(&error));
 	smlErrorDeref(&error);
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&oserror));
-	report_error_on_context(&(env->connectCtx), &oserror, TRUE);
+	report_error_on_context(&(database->connectCtx), &oserror, TRUE);
 }
 
 osync_bool discover(const char *name, void *data, OSyncPluginInfo *info, OSyncError **error)
@@ -730,32 +791,39 @@ void *syncml_init(
 			goto error_free_env;
 	}
 
-	/* prepare the function list for OpenSync */
-	OSyncObjTypeSinkFunctions main_functions;
-	memset(&main_functions, 0, sizeof(main_functions));
-	main_functions.connect = syncml_connect;
-	main_functions.disconnect = disconnect;
 
 	/* Register main sink for connect and disconnect functions */
 	OSyncObjTypeSink *mainsink = osync_objtype_main_sink_new(oerror);
 	if (!mainsink)
 		goto error_free_env;
 
-	osync_objtype_sink_set_functions(mainsink, main_functions, env);
+	/* Set mainsink plugin functions */
+	osync_objtype_sink_set_sync_done_func(mainsink, sync_done);
+	osync_objtype_sink_set_disconnect_func(mainsink, disconnect);
+
+	osync_objtype_sink_set_userdata(mainsink, env);
+
+	/* Prepare state database */
+	osync_objtype_sink_enable_state_db(mainsink, TRUE);
+	if (!osync_objtype_sink_open_state_db(mainsink, info, oerror))
+		goto error_free_env;
+
+	env->stateDB = osync_objtype_sink_get_state_db(mainsink);
+
 	osync_plugin_info_set_main_sink(info, mainsink);
 	osync_objtype_sink_unref(mainsink);
 
-	/* prepare paths for callbacks */
-	env->anchor_path = g_strdup_printf("%s/anchor.db", osync_plugin_info_get_configdir(info));
-	env->devinf_path = g_strdup_printf("%s/devinf.db", osync_plugin_info_get_configdir(info));
-
 	/* set callbacks */
 	smlDataSyncRegisterEventCallback(env->dsObject1, _recv_event, env);
 	smlDataSyncRegisterGetAlertTypeCallback(env->dsObject1, _get_alert_type, env);
 	smlDataSyncRegisterGetAnchorCallback(env->dsObject1, _get_anchor, env);
 	smlDataSyncRegisterSetAnchorCallback(env->dsObject1, _set_anchor, env);
+
+	/* FIXME: disabled devinf callbacks due to missing OSyncCapabilities interface - #1079
 	smlDataSyncRegisterWriteDevInfCallback(env->dsObject1, _write_devinf, env);
 	smlDataSyncRegisterReadDevInfCallback(env->dsObject1, _read_devinf, env);
+	*/
+
 	smlDataSyncRegisterHandleRemoteDevInfCallback(env->dsObject1, _handle_remote_devinf, env);
 	smlDataSyncRegisterChangeStatusCallback(env->dsObject1, _recv_change_status);
 	if (sessionType == SML_SESSION_TYPE_CLIENT)
@@ -764,8 +832,11 @@ void *syncml_init(
 		smlDataSyncRegisterGetAlertTypeCallback(env->dsObject2, _get_alert_type, env);
 		smlDataSyncRegisterGetAnchorCallback(env->dsObject2, _get_anchor, env);
 		smlDataSyncRegisterSetAnchorCallback(env->dsObject2, _set_anchor, env);
+
+		/* FIXME: disabled devinf callbacks due to missing OSyncCapabilities interface - #1079
 		smlDataSyncRegisterWriteDevInfCallback(env->dsObject2, _write_devinf, env);
 		smlDataSyncRegisterReadDevInfCallback(env->dsObject2, _read_devinf, env);
+		*/
 		smlDataSyncRegisterHandleRemoteDevInfCallback(env->dsObject2, _handle_remote_devinf, env);
 		smlDataSyncRegisterChangeStatusCallback(env->dsObject2, _recv_change_status);
 	}
@@ -778,6 +849,7 @@ void *syncml_init(
 	if (sessionType == SML_SESSION_TYPE_CLIENT &&
 	    !ds_client_init_databases(env, info, oerror))
 		goto error_free_env;
+	env->toBeConnected = g_list_length(env->databases);
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
 	return (void *)env;
