@@ -1,6 +1,7 @@
 /*
  * syncml plugin - A syncml plugin for OpenSync
  * Copyright (C) 2005  Armin Bauer <armin.bauer@opensync.org>
+ * Copyright (C) 2008-2009  Michael Bell <michael.bell@opensync.org>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,8 +27,7 @@
 
 #include <opensync/opensync-format.h>
 #include <opensync/opensync-plugin.h>
-#include <opensync/opensync-merger.h>
-#include <opensync/opensync-context.h>
+#include <opensync/opensync-capabilities.h>
 #include <opensync/opensync-data.h>
 #include <opensync/opensync-helper.h>
 #include <opensync/opensync-group.h>
@@ -77,13 +77,6 @@
 
 #define SYNCML_PLUGIN_CONFIG_AUTH_TYPE "AuthType"
 
-typedef enum {
-	OSYNC_PLUGIN_SYNCML_COMMAND_UNKNOWN,
-	OSYNC_PLUGIN_SYNCML_COMMAND_SEND_ALERT,
-	OSYNC_PLUGIN_SYNCML_COMMAND_SEND_SYNC,
-	OSYNC_PLUGIN_SYNCML_COMMAND_RECV_SYNC,
-} OSyncPluginSyncmlDatastoreCommand;
-
 typedef struct SmlPluginEnv {
 	SmlDataSyncObject *dsObject1;
 	SmlDataSyncObject *dsObject2;
@@ -92,12 +85,13 @@ typedef struct SmlPluginEnv {
 	/* libsyncml state management */
 
 	SmlBool abort;
+	SmlBool waitForDisconnect;
 	SmlDataSyncEventType state1;
 	SmlDataSyncEventType state2;
 
 	/* opensync state management */
 
-	OSyncContext *connectCtx;
+	int toBeConnected;
 	OSyncContext *disconnectCtx;
 	GList *databases;
 	unsigned int gotDatabaseCommits; /* only for OMA DS server */
@@ -105,8 +99,7 @@ typedef struct SmlPluginEnv {
 	/* environment data */
 
 	OSyncPluginInfo *pluginInfo;
-	char *anchor_path;
-	char *devinf_path;
+	OSyncSinkStateDB *stateDB;
 
 	GSource *source;
 	GSourceFuncs *source_functions;
@@ -130,11 +123,15 @@ typedef struct SmlDatabase {
 	unsigned int pendingChanges;
 	unsigned int pendingCommits;
 
-	OSyncPluginSyncmlDatastoreCommand command;
-
+	OSyncContext *connectCtx;
 	OSyncContext *syncModeCtx;
 	OSyncContext *getChangesCtx;
 	OSyncContext *commitCtx;
+
+	GMutex *alert_type_mutex;
+	GCond *alert_type_cond;
+
+	osync_bool slowsync;
 } SmlDatabase;
 
 struct commitContext {
@@ -154,9 +151,10 @@ void *syncml_init(
 		OSyncPlugin *plugin,
 		OSyncPluginInfo *info,
 		OSyncError **oerror);
-void syncml_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
-void sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
-void disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx);
+void syncml_connect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data);
+void syncml_connect_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, osync_bool slow_sync, void *data);
+void sync_done(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data);
+void disconnect(OSyncObjTypeSink *sink, OSyncPluginInfo *info, OSyncContext *ctx, void *data);
 
 osync_bool parse_config(
 		SmlTransportType tsp,
@@ -169,13 +167,6 @@ SmlDatabase *syncml_config_parse_database(
 			OSyncPluginResource *res,
 			OSyncError **error);
 
-SmlDatabase *get_database_from_plugin_info(OSyncPluginInfo *info);
-
-/* this is a helper function which adds an object to a GList */
-/* the function guarantees that an object exists only once in */
-/* this GList. No double entrees. */
-GList *g_list_add(GList *databases, void *database);
-
 void finalize(void *data);
 
 void safe_cfree(char **address);
