@@ -24,4 +24,11 @@ SmlDevInf *_read_devinf(
 		void *userdata,
 		SmlError **error);
 
+
+SmlBool _handle_remote_devinf(
+			SmlDataSyncObject *dsObject,
+			SmlDevInf *devinf,
+			void *userdata,
+			SmlError **error);
+
 #endif //_SYNCML_DEVINF_H
