@@ -4,16 +4,24 @@
 #include "syncml_common.h"
 
 void ds_server_get_changeinfo(
-         void *data,
-         OSyncPluginInfo *info,
-         OSyncContext *ctx);
-
-void ds_server_batch_commit(
-         void *data,
+         OSyncObjTypeSink *sink,
          OSyncPluginInfo *info,
          OSyncContext *ctx,
-         OSyncContext **contexts,
-         OSyncChange **changes);
+         osync_bool slow_sync,
+         void *data);
+
+void ds_server_commit(
+         OSyncObjTypeSink *sink,
+         OSyncPluginInfo *info,
+         OSyncContext *ctx,
+         OSyncChange *change,
+         void *data);
+
+void ds_server_committed_all(
+         OSyncObjTypeSink *sink,
+         OSyncPluginInfo *info,
+         OSyncContext *ctx,
+         void *data);
 
 SmlBool ds_server_init_databases(
             SmlPluginEnv *env,
