@@ -62,7 +62,7 @@ error:
 	return NULL;
 }
 
-osync_bool syncml_http_client_discover(void *data, OSyncPluginInfo *info, OSyncError **error)
+osync_bool syncml_http_client_discover(OSyncPluginInfo *info, void *data, OSyncError **error)
 {
 	return discover("syncml-http-client", data, info, error);
 }
