@@ -8,7 +8,8 @@ void *syncml_http_server_init(
 		OSyncError **error);
 
 osync_bool syncml_http_server_discover(
-		void *data, OSyncPluginInfo *info, 
+		OSyncPluginInfo *info,
+		void *data,
 		OSyncError **error);
 
 #endif //_SYNCML_PLUGIN_HTTP_SERVER_H
