@@ -59,7 +59,7 @@ error:
 	return NULL;
 }
 
-osync_bool syncml_obex_client_discover(void *data, OSyncPluginInfo *info, OSyncError **error)
+osync_bool syncml_obex_client_discover(OSyncPluginInfo *info, void *data, OSyncError **error)
 {
 	return discover("syncml-obex-client", data, info, error);
 }
