@@ -8,7 +8,8 @@ void *syncml_obex_client_init(
 		OSyncError **error);
 
 osync_bool syncml_obex_client_discover(
-		void *data, OSyncPluginInfo *info, 
+		OSyncPluginInfo *info,
+		void *data,
 		OSyncError **error);
 
 #endif //_SYNCML_PLUGIN_OBEX_CLIENT_H
